# Copyright (C) 2023-2025 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import pydantic


class Request(pydantic.BaseModel):
    host: str = pydantic.Field(
        default=...,
        description=(
            "The IP address of the client, as seen by the "
            "server."
        )
    )

    url: str = pydantic.Field(
        default=...,
        description="The full URL that was requested."
    )